unit USitePanel;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls;

type

  TSitePanel = class;

  TCustomSite = class(TCustomPanel)
  private
    FDesc:string;
    procedure SetFDesc(const Value: string);
  protected

  public
    ObjectID:integer;
    
    Constructor Create(AOwner:TComponent); Override;
    Destructor  Destroy; Override;
    procedure   Paint; Override;
    procedure   MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); Override;
    procedure   DragOver(Source: TObject; X,Y: Integer; State: TDragState; var Accept: Boolean); Override;
    procedure   DragDrop(Source: TObject; X, Y: Integer); Override;

  published
    property    SiteDesc:string read FDesc write SetFDesc;

  end;

  TOnSelectSite = procedure(Sender:TComponent;SelectObj:TCustomSite) of Object;

  TSitePanel = class(TCustomPanel)
  private
    { Private declarations }
    FOnSelectSite:TOnSelectSite;
    FObjectList:TStrings;
    FImage:TPicture;
    FSiteImage:TPicture;
    FSiteDefWidth:integer;
    FSiteDefHeight:integer;
    procedure SetFloorImage(const Value: TPicture);
    procedure SetFSiteImage(const Value: TPicture);
    procedure setFSiteDefHeight(const Value: integer);
    procedure SetFSiteDefWidth(const Value: integer);
  protected
    { Protected declarations }
  public
    { Public declarations }
    SelectSite:TCustomSite;

    Constructor Create(AOwner:TComponent); Override;
    Destructor  Destroy; Override;
    procedure   Paint; Override;
    procedure   DragOver(Source: TObject; X,Y: Integer; State: TDragState; var Accept: Boolean); Override;
    procedure   DragDrop(Source: TObject; X, Y: Integer); Override;
    function    SiteByIndex(id:integer):TCustomSite;
    function    SiteByName(sName:string):TCustomSite;
    function    SiteByDesc(sDesc:string):TCustomSite;
    function    GetCanvas:TCanvas;
    procedure   AddCustomSite(desc:string);
    procedure   SaveConfigFile;
    procedure   ReadConfigFile;
    procedure   ClearALLCustomSite;
    procedure   DeleteSite(index:integer);
    
  published
    { Published declarations }
    property Align;
    property FloorImage:TPicture read FImage write SetFloorImage;
    property SiteImage:TPicture  read FSiteImage write SetFSiteImage;
    property SiteDefWidth:integer read FSiteDefWidth write SetFSiteDefWidth;
    property SiteDefHeight:integer read FSiteDefHeight write setFSiteDefHeight;

    property OnSelectSite:TOnSelectSite read FOnSelectSite write FOnSelectSite;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Additional', [TSitePanel]);
end;

//TSitePanel
Constructor TSitePanel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  self.Width  := 80;
  self.Height := 80;
  self.FObjectList := TStringList.Create;
  self.FImage := TPicture.Create;
  self.FSiteImage := TPicture.Create;
  self.FSiteDefWidth  := 60;
  self.FSiteDefHeight := 60;
  self.SelectSite := nil;
end;

Destructor TSitePanel.Destroy;
begin
  self.SelectSite := nil;
  if(self.FObjectList <> nil)then
    begin
      self.ClearALLCustomSite;
      self.FObjectList.Free;
    end;
  self.FImage.Free;
  self.FSiteImage.Free;
  inherited Destroy;
end;

procedure TSitePanel.Paint;
var i:integer;
begin
  if(FImage.Graphic = nil)then inherited
  else self.Canvas.StretchDraw(self.Canvas.ClipRect,FImage.Graphic);
  if(self.FObjectList <> nil)then
    begin
      if(self.FObjectList.Count > 0)then
      for i := 0 to (self.FObjectList.Count-1) do
        if(self.FObjectList.Objects[i] <> nil)then
          TCustomSite(self.FObjectList.Objects[i]).Invalidate;
    end;//if...end!!
end;

procedure TSitePanel.SetFloorImage(const Value: TPicture);
begin
  FImage.Assign(Value);
  self.Invalidate;
end;

procedure TSitePanel.DragDrop(Source: TObject; X, Y: Integer);
begin
  TControl(Source).Top  := Y;
  TControl(Source).Left := X;
  self.Invalidate; 
end;

procedure TSitePanel.DragOver(Source: TObject; X, Y: Integer;
  State: TDragState; var Accept: Boolean);
begin
  Accept := True;
end;

function TSitePanel.SiteByIndex(id: integer): TCustomSite;
begin
  result := nil;                                                               
  if(id < 0)then Exit;
  if(self.FObjectList.Count <= 0)then Exit;
  if(id > self.FObjectList.Count-1)then Exit;
  result := TCustomSite(self.FObjectList.Objects[id]);
end;

function TSitePanel.SiteByName(sName: string): TCustomSite;
var i:integer;
begin
  result := nil;
  if(trim(sName) = '')then Exit;
  if(self.FObjectList.Count <= 0)then Exit;
  for i := 0 to (self.FObjectList.Count-1) do
    if(UpperCase(TCustomSite(self.FObjectList.Objects[i]).Name) = UpperCase(sName))then
      result := TCustomSite(self.FObjectList.Objects[i]);
end;

function TSitePanel.SiteByDesc(sDesc:string):TCustomSite;
var i:integer;
begin
  result := nil;
  if(trim(sDesc) = '')then Exit;
  if(self.FObjectList.Count <= 0)then Exit;
  for i := 0 to (self.FObjectList.Count-1) do
    if(UpperCase(TCustomSite(self.FObjectList.Objects[i]).SiteDesc) = UpperCase(sDesc))then
      result := TCustomSite(self.FObjectList.Objects[i]);
end;

function TSitePanel.GetCanvas: TCanvas;
begin
  result := self.Canvas;
end;

procedure TSitePanel.AddCustomSite(desc:string);
var idx:integer;
    comp:TCustomSite;
begin
  comp := nil;
  comp := TCustomSite.Create(nil);
  with comp do
    begin
      Parent := self;
      idx := self.FObjectList.Count;
      ObjectID := idx;
      Name := 'Site'+IntToStr(idx+1);
      SiteDesc := desc;
      width := self.FSiteDefWidth;
      Height := self.FSiteDefHeight;
    end;
  self.FObjectList.AddObject(comp.Name,comp);
  comp := nil;
  self.Invalidate;
end;

procedure TSitePanel.ClearALLCustomSite;
var i:integer;
begin
  if(self.FObjectList <> nil)then
    begin
      if(self.FObjectList.Count > 0)then
      for i := 0 to (self.FObjectList.Count-1) do
        begin
          TCustomSite(self.FObjectList.Objects[i]).Free;
          self.FObjectList.Objects[i] := nil;
        end;//for...i...end!!
      self.FObjectList.Clear;
    end;//if...end!!
end;

procedure TSitePanel.SetFSiteImage(const Value: TPicture);
var i:integer;
begin
  FSiteImage.Assign(Value);
  for i := 0 to self.ControlCount-1 do
    if(self.Controls[i].ClassType = TCustomSite)then
      self.Controls[i].Invalidate;
end;

procedure TSitePanel.setFSiteDefHeight(const Value: integer);
var i:integer;
begin
  FSiteDefHeight := Value;
  for i := 0 to self.ControlCount-1 do
    if(self.Controls[i].ClassType = TCustomSite)then
      self.Controls[i].Invalidate;
end;

procedure TSitePanel.SetFSiteDefWidth(const Value: integer);
var i:integer;
begin
  FSiteDefWidth := Value;
  for i := 0 to self.ControlCount-1 do
    if(self.Controls[i].ClassType = TCustomSite)then
      self.Controls[i].Invalidate;  
end;

procedure TSitePanel.SaveConfigFile;
var sd:TSaveDialog;
    lt:TStrings;
    i:integer;
begin
  try
    lt := TStringList.Create;
    for i := 0 to self.ControlCount-1 do
      if(self.Controls[i].ClassType = TCustomSite)then
        lt.Add(IntToStr(self.Controls[i].Left)
               +','+IntToStr(self.Controls[i].Top)
               +':'+ TCustomSite(self.Controls[i]).SiteDesc);
    sd := TSaveDialog.Create(nil);
    sd.DefaultExt := 'Sit';
    sd.Execute;
    if(trim(sd.FileName) <> '')then
      lt.SaveToFile(sd.FileName);
  finally
    lt.Clear;
    lt.Free;
    sd.Free;
    self.Invalidate;
  end;  
end;

procedure TSitePanel.ReadConfigFile;
var sd:TOpenDialog;
    lt:TStrings;
    i,j,idx:integer;
    str,ts:string;
    xstr,ystr,descstr:string;
    findy,findDesc:boolean;
    comp:TCustomSite;
begin
  try
    lt := TStringList.Create;
    sd := TOpenDialog.Create(nil);
    sd.Filter := '*.Sit';
    sd.Execute;
    if(trim(sd.FileName) <> '')then
      lt.LoadFromFile(sd.FileName);
    for i := 0 to lt.Count-1 do
      begin
        comp := nil;
        findy    := FALSE;
        findDesc := FALSE;
        str := '';
        xstr := '';
        ystr := '';
        descstr := '';
        str := trim(lt[i]);
        if(str <> '')then
        for j := 1 to Length(str) do
          begin
            if(str[j] = ',')then
              begin
                findy := TRUE;
                Continue;
              end
            else
              if(str[j] = ':')then
                begin
                  findDesc := TRUE;
                  Continue;
                end;

            if(findDesc = FALSE)then
              begin
                case findy of
                  FALSE:xstr := xstr+str[j];
                  TRUE :ystr := ystr+str[j];
                end;//case...end!!
              end
            else descstr := descstr + str[j];
          end;//if...end!!

        comp := TCustomSite.Create(nil);
        with comp do
          begin
            Parent   := self;
            Top      := StrToInt(trim(ystr));
            Left     := StrToInt(trim(Xstr));
            width    := self.FSiteDefWidth;
            Height   := self.FSiteDefHeight;
            ObjectID := i;
            Name := 'Site'+IntToStr(i+1);
            SiteDesc := descStr;
           end;
        self.FObjectList.AddObject(comp.Name,comp);   
      end;//for...i...end!!
  finally
    comp := nil;
    if(lt <> nil)then
      begin
        lt.Clear;
        lt.Free;
      end;  
    sd.Free;
  end;
  self.Invalidate;
end;

procedure TSitePanel.DeleteSite(index: integer);
var comp:TCustomSite;
begin
  comp := nil;
  comp := self.SiteByIndex(index);
  if(comp <> nil)then
    begin
      comp.Free;
      comp := nil;
      self.FObjectList.Objects[index] := nil;
    end;  
end;

//TCustomSite
Constructor TCustomSite.Create(AOwner: TComponent);
begin
  inherited Create(AOwner)
end;

Destructor TCustomSite.Destroy;
begin
  inherited Destroy;
end;

procedure TCustomSite.DragDrop(Source: TObject; X, Y: Integer);
begin
  TControl(Source).Top  := self.Top  + Y;
  TControl(Source).Left := self.Left + X;
  self.Invalidate;
  self.Parent.Invalidate;
end;

procedure TCustomSite.DragOver(Source: TObject; X, Y: Integer;
  State: TDragState; var Accept: Boolean);
begin
  Accept := True;
end;

procedure TCustomSite.MouseDown(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
begin
  if(button = mbLeft)then
    begin
      if(TSitePanel(self.Parent).SelectSite <> self)then
        begin
          TSitePanel(self.Parent).SelectSite := self;
          if(Assigned(TSitePanel(self.Parent).FOnSelectSite))then
            TSitePanel(self.Parent).FOnSelectSite(TSitePanel(self.Parent),self);
        end
      else self.BeginDrag(TRUE);
    end;
end;

procedure TCustomSite.Paint;
var txleng:integer;
    tx,ty:integer;
begin
  if(TSitePanel(self.Parent).SiteImage.Graphic <> nil)then
    begin
      self.Canvas.StretchDraw( self.Canvas.ClipRect
                              ,TSitePanel(self.Parent).SiteImage.Graphic);
    end
  else inherited Paint;
  TSitePanel(self.Parent).GetCanvas.Font.Color := ClBlack;
  txleng := TSitePanel(self.Parent).GetCanvas.TextWidth(self.SiteDesc) div 2;
  tx := (self.Left + (self.Width div 2) )- txleng;
  ty := self.Top+self.Height+3;
  TSitePanel(self.Parent).GetCanvas.TextOut(tx,ty,self.SiteDesc);
end;

procedure TCustomSite.SetFDesc(const Value: string);
begin
  FDesc := Value;
end;

end.
